/*

 +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2006 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3.01 of the PHP license,      |
  | that is bundled with this package in the file LICENSE, and is        |
  | available through the world-wide-web at the following url:           |
  | http://www.php.net/license/3_01.txt                                  |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Author:                                                              |
  +----------------------------------------------------------------------+
*/

/* $Id: header,v 1.16.2.1 2006/01/01 12:50:00 sniper Exp $ */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_eseegwtools.h"
#include "balance.h"
#include "devstatus.h"
#include "devtype.h"

#include "stdinc.h"
#include "dvrshm.h"
/* If you declare any globals in php_eseegwtools.h uncomment this:
ZEND_DECLARE_MODULE_GLOBALS(eseegwtools)
*/

/* True global resources - no need for thread safety here */
static int le_eseegwtools;

/* {{{ eseegwtools_functions[]
 *
 * Every user visible function must have an entry in eseegwtools_functions[].
 */
zend_function_entry eseegwtools_functions[] = {
	PHP_FE(confirm_eseegwtools_compiled,	NULL)		/* For testing, remove later. */
	PHP_FE(egts_query_uploadip, NULL)
	PHP_FE(egts_get_aroundcoor, NULL)
	PHP_FE(egts_query_devstatus,NULL)
	PHP_FE(egts_query_devfullinfo,NULL)
	PHP_FE(egts_query_online_dev,NULL)
	PHP_FE(egts_query_dev_list,NULL)
	PHP_FE(egts_query_devtype, NULL)           /*such as black list*/
	PHP_FE(egts_set_devtype,NULL)
	{NULL, NULL, NULL}	/* Must be the last line in eseegwtools_functions[] */
};
/* }}} */

/* {{{ eseegwtools_module_entry
 */
zend_module_entry eseegwtools_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"eseegwtools",
	eseegwtools_functions,
	PHP_MINIT(eseegwtools),
	PHP_MSHUTDOWN(eseegwtools),
	PHP_RINIT(eseegwtools),		/* Replace with NULL if there's nothing to do at request start */
	PHP_RSHUTDOWN(eseegwtools),	/* Replace with NULL if there's nothing to do at request end */
	PHP_MINFO(eseegwtools),
#if ZEND_MODULE_API_NO >= 20010901
	"0.1", /* Replace with version number for your extension */
#endif
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_ESEEGWTOOLS
ZEND_GET_MODULE(eseegwtools)
#endif

#define COND_LOGIN_FAILED (1)
#define COND_ONLINE  (2)
#define COND_OFFLINE (4)
#define COND_HOLDON (8)
#define COND_HOLDOFF (16)
#define COND_ALL (COND_LOGIN_FAILED | COND_ONLINE | COND_OFFLINE | COND_HOLDON | COND_HOLDOFF )
#define TEST_CON(x,c) (((x) & (c)) == (c) )
/* {{{ PHP_INI
 */
/* Remove comments and fill if you need to have entries in php.ini
PHP_INI_BEGIN()
    STD_PHP_INI_ENTRY("eseegwtools.global_value",      "42", PHP_INI_ALL, OnUpdateLong, global_value, zend_eseegwtools_globals, eseegwtools_globals)
    STD_PHP_INI_ENTRY("eseegwtools.global_string", "foobar", PHP_INI_ALL, OnUpdateString, global_string, zend_eseegwtools_globals, eseegwtools_globals)
PHP_INI_END()
*/
/* }}} */

/* {{{ php_eseegwtools_init_globals
 */
/* Uncomment this function if you have INI entries
static void php_eseegwtools_init_globals(zend_eseegwtools_globals *eseegwtools_globals)
{
	eseegwtools_globals->global_value = 0;
	eseegwtools_globals->global_string = NULL;
}
*/
/* }}} */

/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION(eseegwtools)
{
	/* If you have INI entries, uncomment these lines
	ZEND_INIT_MODULE_GLOBALS(eseegwtools, php_eseegwtools_init_globals, NULL);
	REGISTER_INI_ENTRIES();
	*/
	REGISTER_LONG_CONSTANT("EGTS_DEV_COND_LOGIN_FAILED",COND_LOGIN_FAILED,CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("EGTS_DEV_COND_ONLINE",COND_ONLINE,CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("EGTS_DEV_COND_OFFLINE",COND_OFFLINE,CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("EGTS_DEV_COND_HOLDON",COND_HOLDON,CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("EGTS_DEV_COND_HOLDOFF",COND_HOLDOFF,CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("EGTS_DEV_COND_ALL",COND_ALL,CONST_CS | CONST_PERSISTENT);
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MSHUTDOWN_FUNCTION
 */
PHP_MSHUTDOWN_FUNCTION(eseegwtools)
{
	/* uncomment this line if you have INI entries
	UNREGISTER_INI_ENTRIES();
	*/
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request start */
/* {{{ PHP_RINIT_FUNCTION
 */
PHP_RINIT_FUNCTION(eseegwtools)
{
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request end */
/* {{{ PHP_RSHUTDOWN_FUNCTION
 */
PHP_RSHUTDOWN_FUNCTION(eseegwtools)
{
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(eseegwtools)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "eseegwtools support", "enabled");
	php_info_print_table_end();

	/* Remove comments if you have entries in php.ini
	DISPLAY_INI_ENTRIES();
	*/
}
/* }}} */


/* Remove the following function when you have succesfully modified config.m4
   so that your module can be compiled into PHP, it exists only for testing
   purposes. */

/* Every user-visible function in PHP should document itself in the source */
/* {{{ proto string confirm_eseegwtools_compiled(string arg)
   Return a string to confirm that the module is compiled in */
PHP_FUNCTION(confirm_eseegwtools_compiled)
{
	char *arg = NULL;
	int arg_len, len;
	char string[256];

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &arg, &arg_len) == FAILURE) {
		return;
	}

	len = sprintf(string, "Congratulations! You have successfully modified ext/%.78s/config.m4. Module %.78s is now compiled into PHP.", "eseegwtools", arg);
	RETURN_STRINGL(string, len);
}

PHP_FUNCTION(egts_query_uploadip)
{
	unsigned int ip;
	char sIp[16];
	int nLen;

	ip = query_upload_ip();
	nLen = sprintf(sIp,"%u.%u.%u.%u",(unsigned char)(ip & 0xff), (unsigned char)((ip >> 8) & 0xff), (unsigned char)((ip >> 16) & 0xff), (unsigned char)((ip >> 24) & 0xff));

	RETURN_STRINGL(sIp,nLen);
}

PHP_FUNCTION (egts_get_aroundcoor)
{

	double lng;
	double lat;
	long radius;
	//写入参数，失败返回false
	if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC ,"ddl",&lng,&lat,&radius)==FAILURE)
	{
		RETURN_FALSE;
	}

	//参数校验
	if(lng > 180)
	{
		lng=180;
	}
	if(lng <-180)
	{
		lng=-180;
	}
	if(lat >89.99)
	{
		lat=89.99;
	}
	if(lat<-89.99)
	{
		lat=-89.99;
	}
	//The circumference of the earch is 24901miles
	//miles to meters 1609.344, so miles to km is 1.609344
	//radius范围距离，使用km作为单位
	double degree=(24901*1.609344)/360.0;
	double dpm_lat=1/degree;
	double radius_lat=dpm_lat * radius;
	double min_lat=lat-radius_lat;
	double max_lat=lat+radius_lat;
	double mpd_lng=degree*cos(lat*(M_PI/180));
	double dpm_lng=1/mpd_lng;
	double radius_lng=dpm_lng*radius;
	double min_lng=lng-radius_lng;
	double max_lng=lng+radius_lng;

	//添加边界比较
	if(max_lng>180)
		max_lng=max_lng-360;
	if(min_lng<-180)
		min_lng=360+min_lng;
	if(max_lat>90)
		max_lat=90;
	if(min_lat<-90)
		min_lat=-90;
	//返回数组
	array_init(return_value);
	add_assoc_double(return_value,"max_lng",max_lng);
	add_assoc_double(return_value,"min_lng",min_lng);
	add_assoc_double(return_value,"max_lat",max_lat);
	add_assoc_double(return_value,"min_lat",min_lat);
}

struct _tag_online_dev_retriver{
	long start;
	long count;
	long current;
	long condition;
	zval dev_array ;
};


int walk_cb(void *data, void * arg){
	struct _tag_online_dev_retriver * param = (struct _tag_online_dev_retriver *)arg;
	DEV_t * content = (DEV_t *)data;
	// 是否命中条件
	int condition = 0;
	time_t currentt;
	unsigned int uid_length;
	currentt = time(NULL);
	// 命中判读
	if ( (TEST_CON(param->condition,COND_LOGIN_FAILED) && 0 == content->status)
		|| (TEST_CON(param->condition,COND_ONLINE) && 1 == content->status)
		|| (TEST_CON(param->condition,COND_OFFLINE) && 2 == content->status) 
        || (TEST_CON(param->condition, COND_HOLDON) && 3 == content->status)
        || (TEST_CON(param->condition, COND_HOLDOFF) && 4 == content->status)
        ){
		condition = 1;
	} else if ( param->condition == COND_ALL ){
        condition = 1;
    }

	// 命中了则加入返回数组
	if ( 1 == condition){
		if ( param->current >= param->start
				&& param->current < param->start + param->count){
			// insert into dev_array
			uid_length = strlen(content->uid) >= UID_LEN ? uid_length = UID_LEN - 1 : strlen(content->uid);
			add_next_index_stringl(&param->dev_array,content->uid,uid_length);
		}
		param->current ++;
	}

	return 0;

}

PHP_FUNCTION(egts_query_dev_list){
	long start = 0;
	long count = 20;
	long condition = COND_ALL;
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,"|lll",&condition,&start,&count) == FAILURE ){
		RETURN_LONG(-1);
	}

	void * shm = dvr_shm_init();
	if ( NULL == shm || (void *)-1 == shm){
		RETURN_LONG(-2);
	}
	struct _tag_online_dev_retriver p;
	p.start = start;
	p.count = count;
	p.condition = condition;
	p.current = 0;
	array_init(&p.dev_array);

	dvr_shm_walk(shm,walk_cb,&p);

	dvr_shm_deinit(shm);

	object_init(return_value);
	add_property_zval(return_value,"list",&p.dev_array);
	add_property_long(return_value,"total",p.current);
}

PHP_FUNCTION(egts_query_online_dev){
	long start = 0;
	long count = 20;
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,"|ll",&start,&count) == FAILURE ){
		RETURN_LONG(-1);
	}

	void * shm = dvr_shm_init();
	if ( NULL == shm || (void *)-1 == shm){
		RETURN_LONG(-2);
	}
	struct _tag_online_dev_retriver p;
	p.start = start;
	p.count = count;
	p.condition = COND_ONLINE;
	p.current = 0;
	array_init(&p.dev_array);

	dvr_shm_walk(shm,walk_cb,&p);

	dvr_shm_deinit(shm);

	object_init(return_value);
	add_property_zval(return_value,"list",&p.dev_array);
	add_property_long(return_value,"total",p.current);

}

PHP_FUNCTION(egts_query_devfullinfo)
{
	zval * ids;
	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &ids)){
	}

	if ( Z_TYPE_P(ids) != IS_ARRAY){
		RETURN_LONG(-1);
	}

	void * shm = dvr_shm_init();
	if ( NULL == shm || (void *)-1 == shm){
		RETURN_LONG(-2);
	}

	HashTable * tids = Z_ARRVAL(*ids);
	array_init(return_value);
	for ( zend_hash_internal_pointer_reset(tids); zend_hash_has_more_elements(tids) == SUCCESS; zend_hash_move_forward(tids) ){
		zend_string * key;
		zend_ulong keylen;
		unsigned long idx;
		int type;
		zval *pzval, tmpcopy;
		unsigned int uid_length;
		type = zend_hash_get_current_key(tids, &key, &keylen);

		if ( (pzval = zend_hash_get_current_data(tids)) == NULL){
			continue;
		}

		if ( IS_STRING != Z_TYPE_P(pzval) ){
			continue;
		}

		DEV_t *content = (DEV_t *)dvr_shm_get(shm,Z_STRVAL(*pzval));
		if (NULL == content){
			continue;
		}
		zval obj;
		object_init(&obj);
		add_property_long(&obj, "status",  content->status);
		add_property_long(&obj, "nettype", content->nettype);
		add_property_long(&obj, "last", content->last);
		add_property_long(&obj, "ip", content->ip);
		add_property_long(&obj, "port", content->port);
		add_property_long(&obj, "proto", content->proto);
		uid_length = strlen(content->uid) >= UID_LEN ? UID_LEN - 1 : strlen(content->uid);
		add_property_stringl(&obj, "eseeid", content->uid, uid_length);
		add_next_index_zval(return_value,&obj);
	}

	dvr_shm_deinit(shm);
}

PHP_FUNCTION(egts_query_devstatus)
{
	zend_string *id=NULL;
	int arg_len;
#ifdef FAST_ZPP
	ZEND_PARSE_PARAMETERS_START(1,1)
		Z_PARAM_STR(id)
		ZEND_PARSE_PARAMETERS_END();
#else
	if (zend_parse_parameters(ZEND_NUM_ARGS(), "S", &id) == FAILURE) {
		RETURN_LONG(-1);
	}
#endif
	if(ZSTR_LEN(id)==0 || ZSTR_LEN(id)>16)
	{
		RETURN_LONG(-1);
	}

	void * shm = dvr_shm_init();
	if ( NULL == shm || (void *)-1 == shm){
		RETURN_LONG(-2);
	}

	DEV_t *content = (DEV_t *)dvr_shm_get(shm,ZSTR_VAL(id));
	if (NULL == content){
		dvr_shm_deinit(shm);
		RETURN_LONG(-3);
	}
    u_int8_t status = content->status;
	dvr_shm_deinit(shm);
	RETURN_LONG(status);
}


#if 0
PHP_FUNCTION(egts_query_devstatus)
{
	char *arg = NULL;
	int arg_len;
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &arg, &arg_len) == FAILURE) {
		RETURN_LONG(-1);
	}
	if(arg_len==0 || arg_len>16)
	{
		RETURN_LONG(-1);
	}
	char sendbuff[BUFFSIZE] = {0};
	char recvbuff[BUFFSIZE] = {0};
	int sockfd,n;
	struct sockaddr_un client_addr,server_addr;
	socklen_t len;
	sockfd=socket(AF_UNIX,SOCK_DGRAM,0);
	if(sockfd==-1)
	{
		close(sockfd);
		RETURN_LONG (-1);
	}
	memset(&client_addr,0,sizeof(client_addr));
	client_addr.sun_family=AF_UNIX;
	//client_name
	char client_name[32] = {0};
	strcpy(client_addr.sun_path,tmpnam(NULL));
	//len = offsetof(struct sockaddr_un,sun_path) +     strlen(client_name);
	len=sizeof(struct sockaddr_un);
	if(bind(sockfd,(struct sockaddr *)&client_addr,len)==-1)
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
	memset(&server_addr,0,sizeof(server_addr));
	server_addr.sun_family=AF_UNIX;
	strcpy(server_addr.sun_path,SERVER_NAME);
	srandom(time(NULL));
	*(uint32_t*)sendbuff=MAGIC;
	*(uint32_t*)(sendbuff+sizeof(uint32_t))=VERSION;
	*(uint32_t*)(sendbuff+2*sizeof(uint32_t))=CMD_GET_DEV_STATUS;
	*(uint32_t*)(sendbuff+3*sizeof(uint32_t))=20;
	strcpy(sendbuff+4*sizeof(uint32_t),arg);
	*(uint32_t*)(sendbuff+4*sizeof(uint32_t)+16)=random();
	// printf("random is %d\n",random());
	if(sendto(sockfd,sendbuff,36,0, (struct sockaddr*)&server_addr,sizeof(struct sockaddr_un))==-1)
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
	struct timeval tv;
	tv.tv_sec=5;
	tv.tv_usec=0;
	if( setsockopt(sockfd,SOL_SOCKET,SO_RCVTIMEO,&tv,sizeof(tv))==-1 )
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
	len=sizeof(struct sockaddr_un);
	n=recvfrom(sockfd,recvbuff,BUFFSIZE,0, (struct sockaddr*)&server_addr,&len);
	if(n<0)
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
	if(*(uint32_t*)recvbuff!=MAGIC || *(uint32_t*)(recvbuff+sizeof(uint32_t))!=VERSION)
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
	//cmp msg_cmd
	if(*(uint32_t*)(recvbuff+2*sizeof(uint32_t))==CMD_RET_DEV_STATUS)
	{
		//cmp eseeid and random
		if(!strcmp(recvbuff+4*sizeof(uint32_t),sendbuff+4*sizeof(uint32_t)) && *(uint32_t*)(recvbuff+4*sizeof(uint32_t)+16)==*(uint32_t*)(sendbuff+4*sizeof(uint32_t)+16))
		{
			if(*(uint32_t*)(recvbuff+5*sizeof(uint32_t)+16)!=0 && *(uint32_t*)(recvbuff+5*sizeof(uint32_t)+16)!=1)
			{
				close(sockfd);
				unlink(client_addr.sun_path);
				RETURN_LONG (-1);
			}
			else
			{
				close(sockfd);
				unlink(client_addr.sun_path);
				RETURN_LONG (*(uint32_t*)(recvbuff+5*sizeof(uint32_t)+16));
			}

		}
		else
		{

			close(sockfd);
			unlink(client_addr.sun_path);
			RETURN_LONG (-1);

		}


	}
	else if(*(uint32_t*)(recvbuff+2*sizeof(uint32_t))==CMD_FAILURE)
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
	else
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}

}
#endif

PHP_FUNCTION(egts_query_devtype)
{
	char *arg = NULL;
	int arg_len;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &arg, &arg_len) == FAILURE) {
		RETURN_LONG(-1);
	}
	if(arg_len==0 || arg_len>16)
	{
		RETURN_LONG(-1);
	}
	char sendbuff[BUFFSIZE] = {0};
	char recvbuff[BUFFSIZE] = {0};
	int sockfd,n;
	struct sockaddr_un client_addr,server_addr;
	socklen_t len;
	sockfd=socket(AF_UNIX,SOCK_DGRAM,0);
	if(sockfd==-1)
	{
		close(sockfd);
		RETURN_LONG (-1);
	}
	memset(&client_addr,0,sizeof(client_addr));
	client_addr.sun_family=AF_UNIX;
	//client_name
	char client_name[32] = {0};
	strcpy(client_addr.sun_path,tmpnam(NULL));
	//len = offsetof(struct sockaddr_un,sun_path) +     strlen(client_name);
	len=sizeof(struct sockaddr_un);
	if(bind(sockfd,(struct sockaddr *)&client_addr,len)==-1)
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
	memset(&server_addr,0,sizeof(server_addr));
	server_addr.sun_family=AF_UNIX;
	strcpy(server_addr.sun_path,SERVER_NAME);
	srandom(time(NULL));
	*(uint32_t*)sendbuff=MAGIC;
	*(uint32_t*)(sendbuff+sizeof(uint32_t))=VERSION;
	*(uint32_t*)(sendbuff+2*sizeof(uint32_t))=CMD_GET_DEV_TYPE;
	*(uint32_t*)(sendbuff+3*sizeof(uint32_t))=20;
	strcpy(sendbuff+4*sizeof(uint32_t),arg);
	*(uint32_t*)(sendbuff+4*sizeof(uint32_t)+16)=random();
	// printf("random is %d\n",random());
	if(sendto(sockfd,sendbuff,36,0, (struct sockaddr*)&server_addr,sizeof(struct sockaddr_un))==-1)
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
	struct timeval tv;
	tv.tv_sec=5;
	tv.tv_usec=0;
	if( setsockopt(sockfd,SOL_SOCKET,SO_RCVTIMEO,&tv,sizeof(tv))==-1 )
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
	len=sizeof(struct sockaddr_un);
	n=recvfrom(sockfd,recvbuff,BUFFSIZE,0, (struct sockaddr*)&server_addr,&len);
	if(n<0)
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
	if(*(uint32_t*)recvbuff!=MAGIC || *(uint32_t*)(recvbuff+sizeof(uint32_t))!=VERSION)
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
	//cmp msg_cmd
	if(*(uint32_t*)(recvbuff+2*sizeof(uint32_t))==CMD_RET_DEV_TYPE)
	{
		//cmp eseeid and random
		if(!strcmp(recvbuff+4*sizeof(uint32_t),sendbuff+4*sizeof(uint32_t)) && *(uint32_t*)(recvbuff+4*sizeof(uint32_t)+16)==*(uint32_t*)(sendbuff+4*sizeof(uint32_t)+16))
		{
			if(*(uint32_t*)(recvbuff+5*sizeof(uint32_t)+16)!=0 && *(uint32_t*)(recvbuff+5*sizeof(uint32_t)+16)!=1)
			{
				close(sockfd);
				unlink(client_addr.sun_path);
				RETURN_LONG (-1);
			}
			else
			{
				close(sockfd);
				unlink(client_addr.sun_path);
				RETURN_LONG (*(uint32_t*)(recvbuff+5*sizeof(uint32_t)+16));
			}

		}
		else
		{

			close(sockfd);
			unlink(client_addr.sun_path);
			RETURN_LONG (-1);

		}


	}
	else if(*(uint32_t*)(recvbuff+2*sizeof(uint32_t))==CMD_FAILURE)
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
	else
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
}

PHP_FUNCTION(egts_set_devtype)
{
	char  *eseeid = NULL;
	int   id_len;
	long  type;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sl", &eseeid, &id_len,&type) == FAILURE) {
		RETURN_LONG(-1);
	}
	if(id_len==0 || id_len>16)
	{
		RETURN_LONG(-1);
	}
	char sendbuff[BUFFSIZE] = {0};
	char recvbuff[BUFFSIZE] = {0};
	int sockfd,n;
	struct sockaddr_un client_addr,server_addr;
	socklen_t len;
	sockfd=socket(AF_UNIX,SOCK_DGRAM,0);
	if(sockfd==-1)
	{
		close(sockfd);
		RETURN_LONG (-1);
	}
	memset(&client_addr,0,sizeof(client_addr));
	client_addr.sun_family=AF_UNIX;
	//client_name
	char client_name[32] = {0};
	strcpy(client_addr.sun_path,tmpnam(NULL));
	//len = offsetof(struct sockaddr_un,sun_path) +     strlen(client_name);
	len=sizeof(struct sockaddr_un);
	if(bind(sockfd,(struct sockaddr *)&client_addr,len)==-1)
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
	memset(&server_addr,0,sizeof(server_addr));
	server_addr.sun_family=AF_UNIX;
	strcpy(server_addr.sun_path,SERVER_NAME);
	srandom(time(NULL));
	*(uint32_t*)sendbuff=MAGIC;
	*(uint32_t*)(sendbuff+sizeof(uint32_t))=VERSION;
	*(uint32_t*)(sendbuff+2*sizeof(uint32_t))=CMD_SET_DEV_TYPE;
	*(uint32_t*)(sendbuff+3*sizeof(uint32_t))=24;
	strcpy(sendbuff+4*sizeof(uint32_t),eseeid);
	*(uint32_t*)(sendbuff+4*sizeof(uint32_t)+16)=random();
	*(uint32_t*)(sendbuff+4*sizeof(uint32_t)+20)=type;
	// printf("random is %d\n",random());
	if(sendto(sockfd,sendbuff,36,0, (struct sockaddr*)&server_addr,sizeof(struct sockaddr_un))==-1)
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
	struct timeval tv;
	tv.tv_sec=5;
	tv.tv_usec=0;
	if( setsockopt(sockfd,SOL_SOCKET,SO_RCVTIMEO,&tv,sizeof(tv))==-1 )
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
	len=sizeof(struct sockaddr_un);
	n=recvfrom(sockfd,recvbuff,BUFFSIZE,0, (struct sockaddr*)&server_addr,&len);
	if(n<0)
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
	if(*(uint32_t*)recvbuff!=MAGIC || *(uint32_t*)(recvbuff+sizeof(uint32_t))!=VERSION)
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
	//cmp msg_cmd
	if(*(uint32_t*)(recvbuff+2*sizeof(uint32_t))==CMD_RET_SET_DEV_TYPE)
	{
		//cmp eseeid and random
		if(!strcmp(recvbuff+4*sizeof(uint32_t),sendbuff+4*sizeof(uint32_t)) && *(uint32_t*)(recvbuff+4*sizeof(uint32_t)+16)==*(uint32_t*)(sendbuff+4*sizeof(uint32_t)+16))
		{
			if(*(uint32_t*)(recvbuff+5*sizeof(uint32_t)+16)!=0 && *(uint32_t*)(recvbuff+5*sizeof(uint32_t)+16)!=1)
			{
				close(sockfd);
				unlink(client_addr.sun_path);
				RETURN_LONG (-1);
			}
			else
			{
				close(sockfd);
				unlink(client_addr.sun_path);
			}

		}
		else
		{

			close(sockfd);
			unlink(client_addr.sun_path);
			RETURN_LONG (-1);

		}


	}
	else if(*(uint32_t*)(recvbuff+2*sizeof(uint32_t))==CMD_FAILURE)
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
	else
	{
		close(sockfd);
		unlink(client_addr.sun_path);
		RETURN_LONG (-1);
	}
}

/* }}} */
/* The previous line is meant for vim and emacs, so it can correctly fold and
   unfold functions in source code. See the corresponding marks just before
   function definition, where the functions purpose is also documented. Please
   follow this convention for the convenience of others editing your code.
   */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
